part of ali_rtc_fluttify;

class AliRtcEngine extends _Holder with _Community, _Pro {
  static Future<AliRtcEngine> getInstance() async {
    final result = AliRtcEngine._();
    await platform(
      android: (pool) async {
        final context = await android_app_Application.get();
        result._androidEngine = await com_alivc_rtc_AliRtcEngine
            .getInstance__android_content_Context(context);
        await result._androidEngine.setRtcEngineEventListener(
          result._androidEventDelegate,
        );
        await result._androidEngine.setRtcEngineNotify(
          result._androidNotifyDelegate,
        );

        pool.add(context);
      },
      ios: (pool) async {
        result._iosEngine = await ios.AliRtcEngine.sharedInstance_extras(
          result._iosDelegate,
          '',
        );
      },
    );
    return result;
  }

  AliRtcEngine._();
}

mixin _Community on _Holder {
  /// 设置本地画布
  Future<void> setLocalViewConfig(AliVideoCanvas canvas, Track track) async {
    return platform(
      android: (pool) async {
        await _androidEngine.setLocalViewConfig(
          canvas._androidCanvas,
          com_alivc_rtc_AliRtcEngine_AliRtcVideoTrack.values[track.index],
        );
      },
      ios: (pool) async {
        await _iosEngine.setLocalViewConfig_forTrack(
          canvas._iosCanvas,
          ios.AliRtcVideoTrack.values[track.index],
        );
      },
    );
  }

  /// 设置远程画布
  Future<void> setRemoteViewConfig(
    AliVideoCanvas canvas,
    String userId,
    Track track,
  ) async {
    return platform(
      android: (pool) async {
        await _androidEngine.setRemoteViewConfig(
          canvas._androidCanvas,
          userId,
          com_alivc_rtc_AliRtcEngine_AliRtcVideoTrack.values[track.index],
        );
      },
      ios: (pool) async {
        await _iosEngine.setRemoteViewConfig_uid_forTrack(
          canvas._iosCanvas,
          userId,
          ios.AliRtcVideoTrack.values[track.index],
        );
      },
    );
  }

  /// 本地预览。在创建完AliRtcEngine实例后，您可以创建canvas布局进行本地预览视频
  Future<void> startPreview() async {
    return platform(
      android: (pool) async {
        await _androidEngine.startPreview();
      },
      ios: (pool) async {
        await _iosEngine.startPreview();
      },
    );
  }

  /// 取消本地预览
  Future<void> stopPreview() async {
    return platform(
      android: (pool) async {
        await _androidEngine.stopPreview();
      },
      ios: (pool) async {
        await _iosEngine.stopPreview();
      },
    );
  }

  /// 加入频道
  Future<void> joinChannel(AuthInfo info, String username) async {
    return platform(
      android: (pool) async {
        await _androidEngine.joinChannel(await info.toAndroidModel(), username);
      },
      ios: (pool) async {
        await _iosEngine.joinChannel_name_onResult(
          await info.toIOSModel(),
          username,
          (errCode) {},
        );
      },
    );
  }

  /// 离开频道
  Future<void> leaveChannel() async {
    return platform(
      android: (pool) async {
        await _androidEngine.leaveChannel();
      },
      ios: (pool) async {
        await _iosEngine.leaveChannel();
      },
    );
  }

  /// 发布/取消发布本地流
  Future<void> publish() async {
    return platform(
      android: (pool) async {
        await _androidEngine.publish();
      },
      ios: (pool) async {
        await _iosEngine.publish((code) {});
      },
    );
  }

  /// 订阅/取消订阅远程流
  Future<void> subscribe(String userId) async {
    return platform(
      android: (pool) async {
        await _androidEngine.subscribe(userId);
      },
      ios: (pool) async {
        await _iosEngine.subscribe_onResult(
          userId,
          (code, videoTrack, audioTrack) {},
        );
      },
    );
  }

  /// 设置回调
  ///
  /// [onRemoteUserOnLineNotify] 远端用户上线通知
  /// [onRemoteUserOffLineNotify] 远端用户下线通知
  /// [onPublishResult] 发布本地流回调
  /// [onUnpublishResult] 取消发布本地流回调
  Future<void> setEventListener({
    void Function(String uid) onRemoteUserOnLineNotify,
    void Function(String uid) onRemoteUserOffLineNotify,
    void Function(int result) onPublishResult,
    void Function(int result) onUnpublishResult,
  }) async {
    return platform(
      android: (pool) async {
        _androidEventDelegate
          .._onRemoteUserOffLineNotify = onRemoteUserOffLineNotify
          .._onRemoteUserOnLineNotify = onRemoteUserOnLineNotify
          .._onPublishResult = onPublishResult
          .._onUnpublishResult = onUnpublishResult;
      },
      ios: (pool) async {
        _iosDelegate
          .._onRemoteUserOffLineNotify = onRemoteUserOffLineNotify
          .._onRemoteUserOnLineNotify = onRemoteUserOnLineNotify
          .._onPublishResult = onPublishResult
          .._onUnpublishResult = onUnpublishResult;
      },
    );
  }
}

mixin _Pro on _Holder {}

class _Holder {
  com_alivc_rtc_AliRtcEngine _androidEngine;
  ios.AliRtcEngine _iosEngine;

  final _androidEventDelegate = _AndroidEventDelegate();
  final _androidNotifyDelegate = _AndroidNotifyDelegate();
  final _iosDelegate = _IOSDelegate();
}
